<?php

namespace EventPortal\Tests\Unit\Agenda;

use Carbon\Carbon;
use EventPortal\Domain\Agenda\AgendasForDelegate;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Tests\Support\Factories\AgendaFactory;
use EventPortal\Tests\Support\Factories\SessionFactory;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class AgendasForDelegateTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots;

    /** @test */
    public function get_for_delegate()
    {
        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->create();
        $pathway = factory(Pathway::class)->state('default')->create(['event_id' => $event->id]);

        factory(EventDelegate::class)->create([
            'event_id' => $event->id,
            'delegate_id' => $delegate->id,
            'pathway_id' => $pathway->id,
        ]);

        AgendaFactory::make(Carbon::parse('09 September 2020'), $event)
            ->createSession($pathway, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => Carbon::parse('09 September 2020 09:00am'),
                        'ending_at' => Carbon::parse('09 September 2020 10:00am'),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'One',
                    ])
                    ->createVenue([
                        'name' => 'Venue one',
                    ]);
            });

        $this->assertMatchesSnapshot(AgendasForDelegate::get($delegate, $event));
    }

    /** @test */
    public function gets_multiple_days()
    {
        $event = factory(Event::class)->create();
        $delegate = factory(Delegate::class)->create();
        $pathway = factory(Pathway::class)->state('default')->create(['event_id' => $event->id]);

        factory(EventDelegate::class)->create([
            'event_id' => $event->id,
            'delegate_id' => $delegate->id,
            'pathway_id' => $pathway->id,
        ]);

        AgendaFactory::make(Carbon::parse('09 September 2020'), $event)
            ->createSession($pathway, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => Carbon::parse('09 September 2020 09:00am'),
                        'ending_at' => Carbon::parse('09 September 2020 10:00am'),
                        'name' => 'Session 1',
                        'description' => 'Description for session 1',
                        'location' => 'Studio 1',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'One',
                    ])
                    ->createVenue([
                        'name' => 'Venue one',
                    ]);
            });

        AgendaFactory::make(Carbon::parse('10 September 2020'), $event)
            ->createSession($pathway, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => Carbon::parse('10 September 2020 09:00am'),
                        'ending_at' => Carbon::parse('10 September 2020 10:00am'),
                        'name' => 'Session for day two',
                        'description' => 'Description for session',
                        'location' => 'Studio 2',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'Two',
                    ])
                    ->createVenue([
                        'name' => 'Venue two',
                    ]);
            });

        $data = AgendasForDelegate::get($delegate, $event);

        $this->assertMatchesSnapshot($data);
    }

    /** @test */
    public function get_correct_sessions_for_pathway()
    {
        $event = factory(Event::class)->create();

        $delegateA = factory(Delegate::class)->create();
        $pathwayA = factory(Pathway::class)->state('default')->create(['event_id' => $event->id]);

        factory(EventDelegate::class)->create([
            'event_id' => $event->id,
            'delegate_id' => $delegateA->id,
            'pathway_id' => $pathwayA->id,
        ]);

        $delegateB = factory(Delegate::class)->create();
        $pathwayB = factory(Pathway::class)->create(['event_id' => $event->id]);

        factory(EventDelegate::class)->create([
            'event_id' => $event->id,
            'delegate_id' => $delegateB->id,
            'pathway_id' => $pathwayB->id,
        ]);

        AgendaFactory::make(Carbon::parse('09 September 2020'), $event)
            ->createSession($pathwayA, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => Carbon::parse('09 September 2020 09:00am'),
                        'ending_at' => Carbon::parse('09 September 2020 10:00am'),
                        'name' => 'Normal Session',
                        'description' => 'Normal Session',
                        'location' => 'Studio 1',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'One',
                    ])
                    ->createVenue([
                        'name' => 'Venue one',
                    ]);
            })
            ->createSession($pathwayB, static function (SessionFactory $sessionFactory): void {
                $sessionFactory
                    ->create([
                        'starting_at' => Carbon::parse('09 September 2020 09:00am'),
                        'ending_at' => Carbon::parse('09 September 2020 10:00am'),
                        'name' => 'Doctor Session',
                        'description' => 'Doctor Session Description',
                        'location' => 'Studio 1',
                    ])
                    ->createSpeakerAndUser([], [
                        'first_name' => 'Speaker',
                        'last_name' => 'One',
                    ])
                    ->createVenue([
                        'name' => 'Venue one',
                    ]);
            });

        $dataForDelegateA = AgendasForDelegate::get($delegateA, $event);
        $this->assertMatchesSnapshot($dataForDelegateA);

        $dataForDelegateB = AgendasForDelegate::get($delegateB, $event);
        $this->assertMatchesSnapshot($dataForDelegateB);
    }
}
