<?php

namespace EventPortal\Tests\Unit\Util;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Util\Render;
use EventPortal\Seeds\RoleSeeder;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class RenderTest extends TestCase
{
    use MatchesSnapshots, RefreshDatabase;

    public function setUp(): void
    {
        parent::setUp();
        $this->withoutMiddleware();

        $this->pageHeader = [
            "title" => "Dashboard",
            "description" => "A home page",
            "howToText" => 'Text that isn\'t null',
            "howToVideo" => null,
        ];

        config(['page-headers.admin.dashboard.index' => $this->pageHeader]);

        (new RoleSeeder())->run();
        $this->actingAsAdmin();
    }

    /** @test */
    public function can_get_page_config(): void
    {
        $render = new Render('Admin/Dashboard', 'admin');
        $viewdata = $render
            ->key('dashboard.index')
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_extend_config(): void
    {
        $render = new Render('Some/Page', 'admin');
        $viewdata = $render
            ->key('dashboard.index')
            ->attributes(['extraData' => [
                'first' => 'Wow',
                'second' => 'noice',
            ]])->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_override_title(): void
    {
        $render = new Render('Some/Page', 'admin');
        $viewdata = $render
            ->key('dashboard.index')
            ->withTitle('Custom Title Override')
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_form(): void
    {
        $form = Form::make('event.create')->model(new Event)->render();

        $render = new Render('Some/Page', 'admin');
        $viewdata = $render
            ->key('dashboard.index')
            ->withForm($form)
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_multiple_forms(): void
    {
        $form = Form::make('event.create')->model(new Event)->render();

        $render = new Render('Some/Page', 'admin');
        $viewdata = $render
            ->key('dashboard.index')
            ->withForms([
                'formA' => $form,
                'formB' => $form,
            ])
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_include_data(): void
    {
        $render = new Render('Some/Page', 'admin');
        $viewdata = $render
            ->key('dashboard.index')
            ->withData([
                'first' => 'baked',
                'second' => 'beans',
                'third' => [
                    [
                        'name' => 'beans',
                        'type' => 'baked',
                    ],
                    [
                        'name' => 'beans',
                        'type' => 'kidney',
                    ],
                ],
            ])
            ->getData();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }

    /** @test */
    public function can_set_meta_data(): void
    {
        $render = new Render('Some/Page', 'admin');
        $viewdata = $render
            ->withMeta('Custom Meta Title', 'Custom Meta Description')
            ->getMeta();

        $this->assertMatchesJsonSnapshot(json_encode($viewdata));
    }
}
